@@ -35,7 +35,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: plugin_defaulttarget.c,v 1.7 2009/02/22 15:32:55 hb9xar Exp $";
+static char const ident[]="$Id: plugin_defaulttarget.c 440 2010-01-07 11:30:27Z hb9xar $";
 
 /* Plug-in identification */
 static char name[]="plugin_defaulttarget";
@@ -52,8 +52,8 @@ static struct plugin_config {
 
 /* Instructions for config parser */
 static cfgopts_t plugin_cfg_opts[] = {
-   { "plugin_defaulttarget_target",      TYP_STRING, &plugin_cfg.target },
-   { "plugin_defaulttarget_log",         TYP_INT4,   &plugin_cfg.log },
+   { "plugin_defaulttarget_target",      TYP_STRING, &plugin_cfg.target,	{0, NULL} },
+   { "plugin_defaulttarget_log",         TYP_INT4,   &plugin_cfg.log,		{0, NULL} },
    {0, 0, 0}
 };
 /* local storage */
