@@ -36,7 +36,7 @@
 #include "log.h"
 
 
-static char const ident[]="$Id: plugin_demo.c,v 1.5 2009/02/22 15:32:55 hb9xar Exp $";
+static char const ident[]="$Id: plugin_demo.c 440 2010-01-07 11:30:27Z hb9xar $";
 
 /* Plug-in identification */
 static char name[]="plugin_demo";
@@ -52,7 +52,7 @@ static struct plugin_config {
 
 /* Instructions for config parser */
 static cfgopts_t plugin_cfg_opts[] = {
-   { "plugin_demo_string",      TYP_STRING, &plugin_cfg.string },
+   { "plugin_demo_string",      TYP_STRING, &plugin_cfg.string,			{0, NULL} },
    {0, 0, 0}
 };
 
