@@ -35,7 +35,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: plugin_fix_bogus_via.c,v 1.3 2009/02/22 15:32:55 hb9xar Exp $";
+static char const ident[]="$Id: plugin_fix_bogus_via.c 439 2010-01-07 11:29:00Z hb9xar $";
 
 /* Plug-in identification */
 static char name[]="plugin_fix_bogus_via";
@@ -51,7 +51,7 @@ static struct plugin_config {
 
 /* Instructions for config parser */
 static cfgopts_t plugin_cfg_opts[] = {
-   { "plugin_fix_bogus_via_networks",      TYP_STRING, &plugin_cfg.networks },
+   { "plugin_fix_bogus_via_networks",      TYP_STRING, &plugin_cfg.networks,	{0, NULL} },
    {0, 0, 0}
 };
 
