@@ -37,7 +37,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: plugin_shortdial.c,v 1.13 2009/03/24 17:38:33 hb9xar Exp $";
+static char const ident[]="$Id: plugin_shortdial.c 466 2010-10-13 22:48:27Z hb9xar $";
 
 /* Plug-in identification */
 static char name[]="plugin_shortdial";
@@ -54,8 +54,8 @@ static struct plugin_config {
 } plugin_cfg;
 /* Instructions for config parser */
 static cfgopts_t plugin_cfg_opts[] = {
-   { "plugin_shortdial_akey",   TYP_STRING, &plugin_cfg.shortdial_akey },
-   { "plugin_shortdial_entry",  TYP_STRINGA,&plugin_cfg.shortdial_entry },
+   { "plugin_shortdial_akey",   TYP_STRING, &plugin_cfg.shortdial_akey,		{0, NULL} },
+   { "plugin_shortdial_entry",  TYP_STRINGA,&plugin_cfg.shortdial_entry,	{0, NULL} },
    {0, 0, 0}
 };
 
@@ -136,7 +136,7 @@ static int plugin_shortdial(sip_ticket_t *ticket) {
       return STS_SUCCESS; /* ignore */
 
    shortcut_no = atoi(&(req_url->username[1]));
-   if ((shortcut_no <= 0) || (shortcut_no >= LONG_MAX)) return STS_SUCCESS; /* not a number */
+   if ((shortcut_no <= 0) || (shortcut_no >= INT_MAX)) return STS_SUCCESS; /* not a number */
 
    /* requested number is not defined (out of range) */
    if (shortcut_no > plugin_cfg.shortdial_entry.used) {
@@ -179,7 +179,7 @@ static int plugin_shortdial_redirect(sip_ticket_t *ticket, int shortcut_no) {
    char *new_to_host=NULL;
    int  i;
    size_t username_len;
-   size_t host_len;
+   size_t host_len=0;
    osip_contact_t *contact = NULL;
 
    new_to_user=plugin_cfg.shortdial_entry.string[shortcut_no-1];
