@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2002-2008  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2002-2009  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
     
@@ -37,7 +37,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: proxy.c,v 1.117 2009/07/30 17:20:55 hb9xar Exp $";
+static char const ident[]="$Id: proxy.c 456 2010-03-29 17:28:12Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;	/* defined in siproxd.c */
@@ -95,6 +95,16 @@ int proxy_request (sip_ticket_t *ticket) {
 
    request=ticket->sipmsg;
 
+   /*
+    * RFC&&&&
+    * add a received= parameter to the topmost Via header. Used for TCP
+    * connections - send answer within the existing TCP connection back
+    * to client.
+    */
+   if (ticket->protocol == PROTO_TCP) {
+      sip_add_received_param(ticket);
+   }
+
    /*
     * RFC 3261, Section 16.4
     * Proxy Behavior - Route Information Preprocessing
@@ -153,6 +163,10 @@ int proxy_request (sip_ticket_t *ticket) {
          /* Rewrite the body */
          sts = proxy_rewrite_invitation_body(ticket, DIR_INCOMING);
 
+      } else if (MSG_IS_UPDATE(request)) {
+         /* Rewrite the body */
+         sts = proxy_rewrite_invitation_body(ticket, DIR_INCOMING);
+
       }
 sts=sip_obscure_callid(ticket);
       break;
@@ -203,6 +217,8 @@ sts=sip_obscure_callid(ticket);
          sts = proxy_rewrite_invitation_body(ticket, DIR_OUTGOING);
       } else if (MSG_IS_ACK(request) || MSG_IS_PRACK(request)) {
          sts = proxy_rewrite_invitation_body(ticket, DIR_OUTGOING);
+      } else if (MSG_IS_UPDATE(request)) {
+         sts = proxy_rewrite_invitation_body(ticket, DIR_OUTGOING);
       }
 
       /* if this is CANCEL/BYE request, stop RTP proxying */
@@ -337,29 +353,6 @@ sts=sip_obscure_callid(ticket);
  * 2) fixed outbound proxy
  * 3) SIP URI
  */
-   /*
-    * fixed or domain outbound proxy defined ?
-    */
-// let's try with Route header first
-#if 0
-   if ((type == REQTYP_OUTGOING) &&
-       (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
-      DEBUGC(DBCLASS_PROXY, "proxy_request: have outbound proxy %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
-   /*
-    * Route present?
-    * If so, fetch address from topmost Route: header and remove it.
-    */
-   } else if ((type == REQTYP_OUTGOING) && 
-              (!osip_list_eol(&(request->routes), 0))) {
-      sts=route_determine_nexthop(ticket, &sendto_addr, &port);
-      if (sts == STS_FAILURE) {
-         DEBUGC(DBCLASS_PROXY, "proxy_request: route_determine_nexthop failed");
-         return STS_FAILURE;
-      }
-      DEBUGC(DBCLASS_PROXY, "proxy_request: have Route header to %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
-#else
    /*
     * Route present?
     * If so, fetch address from topmost Route: header and remove it.
@@ -373,11 +366,13 @@ sts=sip_obscure_callid(ticket);
       }
       DEBUGC(DBCLASS_PROXY, "proxy_request: have Route header to %s:%i",
              utils_inet_ntoa(sendto_addr), port);
+   /*
+    * fixed or domain outbound proxy defined ?
+    */
    } else if ((type == REQTYP_OUTGOING) &&
        (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_request: have outbound proxy %s:%i",
              utils_inet_ntoa(sendto_addr), port);
-#endif
    /*
     * destination from SIP URI
     */
@@ -535,7 +530,8 @@ sts=sip_obscure_callid(ticket);
        *                       start RTP proxy stream(s). In case
        *                       of a negative answer, stop RTP stream
        */
-      if (MSG_IS_RESPONSE_FOR(response,"INVITE")) {
+      if ((MSG_IS_RESPONSE_FOR(response,"INVITE")) ||
+          (MSG_IS_RESPONSE_FOR(response,"UPDATE"))) {
          /* positive response, start RTP stream */
          if ((MSG_IS_STATUS_1XX(response)) || 
               (MSG_IS_STATUS_2XX(response))) {
@@ -622,7 +618,8 @@ sts=sip_obscure_callid(ticket);
        *
        * In case of a negative answer, stop RTP stream
        */
-      if (MSG_IS_RESPONSE_FOR(response,"INVITE")) {
+      if ((MSG_IS_RESPONSE_FOR(response,"INVITE")) ||
+          (MSG_IS_RESPONSE_FOR(response,"UPDATE"))) {
          /* positive response, start RTP stream */
          if ((MSG_IS_STATUS_1XX(response)) || 
               (MSG_IS_STATUS_2XX(response))) {
@@ -670,18 +667,17 @@ sts=sip_obscure_callid(ticket);
     * Determine Next-Hop Address
     */
 /*&&&& priority probably should be:
+ * 0) rport=;received= header (TCP only for now)
  * 1) Route header
  * 2) fixed outbound proxy
  * 3) Via header
  */
    /*
-    * check if we need to send to an outbound proxy
+    * IF TCP, check for rport=x;received=y parameters in VIA
     */
-// let's try with Route header first
-#if 0
-   if ((type == RESTYP_OUTGOING) &&
-       (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
-      DEBUGC(DBCLASS_PROXY, "proxy_response: have outbound proxy %s:%i",
+   if ((ticket->protocol == PROTO_TCP) && 
+       (sip_get_received_param(ticket,  &sendto_addr, &port) == STS_SUCCESS)) {
+      DEBUGC(DBCLASS_PROXY, "proxy_response: have received/rport to %s:%i",
              utils_inet_ntoa(sendto_addr), port);
    /*
     * Route present?
@@ -696,25 +692,13 @@ sts=sip_obscure_callid(ticket);
       }
       DEBUGC(DBCLASS_PROXY, "proxy_response: have Route header to %s:%i",
              utils_inet_ntoa(sendto_addr), port);
-#else
    /*
-    * Route present?
-    * If so, fetch address from topmost Route: header and remove it.
+    * check if we need to send to an outbound proxy
     */
-   if ((type == RESTYP_OUTGOING) && 
-              (!osip_list_eol(&(response->routes), 0))) {
-      sts=route_determine_nexthop(ticket, &sendto_addr, &port);
-      if (sts == STS_FAILURE) {
-         DEBUGC(DBCLASS_PROXY, "proxy_response: route_determine_nexthop failed");
-         return STS_FAILURE;
-      }
-      DEBUGC(DBCLASS_PROXY, "proxy_response: have Route header to %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
    } else if ((type == RESTYP_OUTGOING) &&
        (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_response: have outbound proxy %s:%i",
              utils_inet_ntoa(sendto_addr), port);
-#endif
    } else {
       /* get target address and port from VIA header */
       via = (osip_via_t *) osip_list_get (&(response->vias), 0);
@@ -779,6 +763,7 @@ int proxy_rewrite_invitation_body(sip_ticket_t *ticket, int direction){
    sdp_connection_t *sdp_conn;
    sdp_media_t *sdp_med;
    int rtp_direction=0;
+   int call_direction=0;
    int have_c_media=0;
    int isrtp = 0 ;
 
@@ -822,11 +807,15 @@ if (configuration.debuglevel)
    size_t tmplen;
    sts = osip_message_get_body(mymsg, 0, &body);
    sts = sip_body_to_str(body, &tmp, &tmplen);
-   osip_content_length_to_str(mymsg->content_length, &tmp2);
-   DEBUG("Body before rewrite (may be truncated) - (clen=%s, strlen=%ld):\n%s\n----",
-         tmp2, (long)tmplen, tmp);
-   osip_free(tmp);
-   osip_free(tmp2);
+   if (sts == 0) {
+      osip_content_length_to_str(mymsg->content_length, &tmp2);
+      DEBUG("Body before rewrite (may be truncated) - (clen=%s, strlen=%ld):\n%s\n----",
+            tmp2, (long)tmplen, tmp);
+      osip_free(tmp);
+      osip_free(tmp2);
+   } else {
+      DEBUG("Body before rewrite: failed to decode!");
+   }
 }
 
    /*
@@ -850,18 +839,22 @@ if (configuration.debuglevel)
    if (MSG_IS_REQUEST(mymsg)) {
       if (direction == DIR_INCOMING) {
          memcpy(&map_addr, &inside_addr, sizeof (map_addr));
-         rtp_direction = DIR_OUTGOING;
+         rtp_direction  = DIR_OUTGOING;
+         call_direction = DIR_INCOMING;
       } else {
          memcpy(&map_addr, &outside_addr, sizeof (map_addr));
-         rtp_direction = DIR_INCOMING;
+         rtp_direction  = DIR_INCOMING;
+         call_direction = DIR_OUTGOING;
       }
    } else /* MSG_IS_REPONSE(mymsg) */ {
       if (direction == DIR_INCOMING) {
          memcpy(&map_addr, &inside_addr, sizeof (map_addr));
-         rtp_direction = DIR_OUTGOING;
+         rtp_direction  = DIR_OUTGOING;
+         call_direction = DIR_OUTGOING;
       } else {
          memcpy(&map_addr, &outside_addr, sizeof (map_addr));
-         rtp_direction = DIR_INCOMING;
+         rtp_direction  = DIR_INCOMING;
+         call_direction = DIR_INCOMING;
       }
    }
 
@@ -1104,9 +1097,12 @@ if (configuration.debuglevel)
                }
             }
 
+            /*
+             * Start the RTP stream
+             */
             sts = rtp_start_fwd(osip_message_get_call_id(mymsg),
                                 client_id,
-                                rtp_direction,
+                                rtp_direction, call_direction,
                                 media_stream_no,
                                 map_addr, &map_port,
                                 addr_media, msg_port,
@@ -1166,11 +1162,15 @@ if (configuration.debuglevel)
    size_t tmplen;
    sts = osip_message_get_body(mymsg, 0, &body);
    sts = sip_body_to_str(body, &tmp, &tmplen);
-   osip_content_length_to_str(mymsg->content_length, &tmp2);
-   DEBUG("Body after rewrite (may be truncated) - (clen=%s, strlen=%ld):\n%s\n----",
-         tmp2, (long)tmplen, tmp);
-   osip_free(tmp);
-   osip_free(tmp2);
+   if (sts == 0) {
+      osip_content_length_to_str(mymsg->content_length, &tmp2);
+      DEBUG("Body after rewrite (may be truncated) - (clen=%s, strlen=%ld):\n%s\n----",
+            tmp2, (long)tmplen, tmp);
+      osip_free(tmp);
+      osip_free(tmp2);
+   } else {
+      DEBUG("Body after rewrite: failed to decode!");
+   }
 }
    return STS_SUCCESS;
 }
