@@ -35,7 +35,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: register.c,v 1.62 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: register.c 419 2009-02-28 23:06:22Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -57,6 +57,7 @@ void register_init(void) {
    int sts, i;
    size_t len;
    char buff[128];
+   char *c;
 
    memset(urlmap, 0, sizeof(urlmap));
 
@@ -73,7 +74,7 @@ void register_init(void) {
       } else {
          /* read the url table from file */
          for (i=0;i < URLMAP_SIZE; i++) {
-            fgets(buff, sizeof(buff), stream);
+            c=fgets(buff, sizeof(buff), stream);
             sts=sscanf(buff, "***:%i:%i", &urlmap[i].active, &urlmap[i].expires);
             if (sts == 0) break; /* format error */
             if (urlmap[i].active) {
@@ -82,7 +83,7 @@ void register_init(void) {
                osip_uri_init(&urlmap[i].reg_url);
 
                #define R(X) {\
-               fgets(buff, sizeof(buff), stream);\
+               c=fgets(buff, sizeof(buff), stream);\
                buff[sizeof(buff)-1]='\0';\
                if (strchr(buff, 10)) *strchr(buff, 10)='\0';\
                if (strchr(buff, 13)) *strchr(buff, 13)='\0';\
@@ -293,12 +294,12 @@ int register_client(sip_ticket_t *ticket, int force_lcl_masq) {
    if (expires_param && expires_param->gvalue) {
       /* get expires from contact Header */
       expires=atoi(expires_param->gvalue);
-      if ((expires < 0) || (expires >= LONG_MAX))
+      if ((expires < 0) || (expires >= UINT_MAX ))
          expires=configuration.default_expires;
    } else if (expires_hdr && expires_hdr->hvalue) {
       /* get expires from expires Header */
       expires=atoi(expires_hdr->hvalue);
-      if ((expires < 0) || (expires >= LONG_MAX))
+      if ((expires < 0) || (expires >= UINT_MAX ))
          expires=configuration.default_expires;
    } else {
       char tmp[16];
@@ -665,12 +666,12 @@ int register_set_expire(sip_ticket_t *ticket) {
       if (expires_param && expires_param->gvalue) {
          /* get expires from contact Header */
          expires=atoi(expires_param->gvalue);
-         if ((expires < 0) || (expires >= LONG_MAX))
+         if ((expires < 0) || (expires >= UINT_MAX ))
             expires=configuration.default_expires;
       } else if (expires_hdr && expires_hdr->hvalue) {
          /* get expires from expires Header */
          expires=atoi(expires_hdr->hvalue);
-         if ((expires < 0) || (expires >= LONG_MAX))
+         if ((expires < 0) || (expires >= UINT_MAX ))
             expires=configuration.default_expires;
       }
 
