@@ -33,7 +33,9 @@
 
 #include "log.h"
 
-static char const ident[]="$Id: resolve.c,v 1.9 2009/02/22 19:22:53 hb9xar Exp $";
+#define USE_NAPTR	0
+
+static char const ident[]="$Id: resolve.c 442 2010-01-07 11:31:17Z hb9xar $";
 
 
 /* local functions */
@@ -54,7 +56,7 @@ int resolve_SRV(char *name, char *dname, int dnamelen, int *port) {
    return _resolve(name, C_IN, T_SRV, dname, dnamelen, port);
 }
 
-#if 0
+#if USE_NAPTR
 /*
  * perform a NAPTR lookup
  *
@@ -186,7 +188,7 @@ Currently just the first (lowest prio, highest weight) entry is returned.
                   xptr+=j;
                }
             }
-#if 0
+#if USE_NAPTR
          } else if( ty == T_NAPTR ) {
             DEBUGC(DBCLASS_DNS, "_resolve: A - type NAPTR");
             usp = (unsigned short *)xptr;
