@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2003-2008  Thomas Ries <tries@gmx.net>
+    Copyright (C) 2003-2009  Thomas Ries <tries@gmx.net>
 
     This file is part of Siproxd.
 
@@ -46,7 +46,7 @@
    #include "dejitter.h"
 #endif
 
-static char const ident[]="$Id: rtpproxy_relay.c,v 1.57 2009/07/30 17:02:28 hb9xar Exp $";
+static char const ident[]="$Id: rtpproxy_relay.c 482 2011-06-12 18:45:17Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -78,7 +78,7 @@ static void sighdl_alm(int sig) {/* just wake up from select() */};
 static void *rtpproxy_main(void *i);
 static void rtpproxy_kill( void );
 static int  rtp_recreate_fdset(void);
-static void match_socket (int rtp_proxytable_idx);
+static int  match_socket (int rtp_proxytable_idx);
 static void error_handler (int rtp_proxytable_idx, int socket_type);
 
 
@@ -92,6 +92,8 @@ int rtp_relay_init( void ) {
    int sts;
    int arg=0;
    struct sigaction sigact;
+   pthread_attr_t attr;
+   size_t stacksize;
 
 #ifdef USE_DEJITTER
    dejitter_init();
@@ -113,8 +115,28 @@ int rtp_relay_init( void ) {
    sigact.sa_flags=0;
    sigaction(SIGALRM, &sigact, NULL);
 
+   pthread_attr_init(&attr);
+   pthread_attr_init(&attr);
+   pthread_attr_getstacksize (&attr, &stacksize);
+   INFO("Current thread stacksize is %i kB",stacksize/1024);   
+
+   /* experimental feature:
+    * reduce the thread stack size to reduce the overall
+    * memory footprint of siproxd on embedded systems
+    *
+    * Use at your own risk! 
+    * Too small stack size may lead to unexplainable crashes!
+    * Improper use may make siproxd eat your dog and vandalize
+    * your garden.
+    */
+   if (configuration.thread_stack_size > 0) {
+      stacksize = configuration.thread_stack_size*1024;
+      pthread_attr_setstacksize (&attr, stacksize);
+      INFO("Setting new thread stacksize to %i kB",stacksize/1024);   
+   }
+
    DEBUGC(DBCLASS_RTP,"create thread");
-   sts=pthread_create(&rtpproxy_tid, NULL, rtpproxy_main, (void *)&arg);
+   sts=pthread_create(&rtpproxy_tid, &attr, rtpproxy_main, (void *)&arg);
    DEBUGC(DBCLASS_RTP,"created, sts=%i", sts);
 
    /* set realtime scheduling - if started by root */
@@ -262,9 +284,9 @@ static void *rtpproxy_main(void *arg) {
              * have got some data in it (count > 0)
              */
             if (count > 0) {
-               /* find the corresponding TX socket */
-               if (rtp_proxytable[i].rtp_con_tx_sock == 0) match_socket(i);
-
+               /* send only if I have the matching TX socket, otherwise throw away.
+                * this requires a full 2-way communication to be set up for each
+                * RTP stream... */
                if (rtp_proxytable[i].rtp_con_tx_sock != 0) {
                   struct sockaddr_in dst_addr;
 
@@ -314,9 +336,9 @@ static void *rtpproxy_main(void *arg) {
              * have got some data in it (count > 0)
              */
             if (count > 0) {
-               /* find the corresponding TX socket */
-               if (rtp_proxytable[i].rtp_tx_sock == 0) match_socket(i);
-
+               /* send only if I have the matching TX socket, otherwise throw away.
+                * this requires a full 2-way communication to be set up for each
+                * RTP stream... */
                if (rtp_proxytable[i].rtp_tx_sock != 0) {
                   struct sockaddr_in dst_addr;
 #ifdef USE_DEJITTER
@@ -487,7 +509,7 @@ static void *rtpproxy_main(void *arg) {
  *	STS_FAILURE on error
  */
 int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
-                         int rtp_direction,
+                         int rtp_direction, int call_direction,
                          int media_stream_no, struct in_addr local_ipaddr,
                          int *local_port, struct in_addr remote_ipaddr,
                          int remote_port, int dejitter) {
@@ -526,9 +548,10 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
    }
 
    DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: starting RTP proxy "
-          "stream for: CallID=%s@%s [Client-ID=%s] (%s) #=%i",
+          "stream for: CallID=%s@%s [Client-ID=%s] (%s,%s) #=%i",
           callid->number, callid->host, client_id.idstring,
           ((rtp_direction == DIR_INCOMING) ? "incoming RTP" : "outgoing RTP"),
+          ((call_direction == DIR_INCOMING) ? "incoming Call" : "outgoing Call"),
           media_stream_no);
 
    /* lock mutex */
@@ -660,10 +683,10 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
       /* port is available, try to allocate */
       if (j == RTPPROXY_SIZE) {
          port=i;
-         sock=sockbind(local_ipaddr, port, 0);	/* RTP */
+         sock=sockbind(local_ipaddr, port, PROTO_UDP, 0);	/* RTP */
 
          if (sock) {
-            sock_con=sockbind(local_ipaddr, port+1, 0);	/* RTCP */
+            sock_con=sockbind(local_ipaddr, port+1, PROTO_UDP, 0);	/* RTCP */
             /* if success break, else try further on */
             if (sock_con) break;
             sts = close(sock);
@@ -737,6 +760,7 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
    memcpy(&rtp_proxytable[freeidx].client_id, &client_id, sizeof(client_id_t));
 
    rtp_proxytable[freeidx].direction = rtp_direction;
+   rtp_proxytable[freeidx].call_direction = call_direction;
    rtp_proxytable[freeidx].media_stream_no = media_stream_no;
    memcpy(&rtp_proxytable[freeidx].local_ipaddr,
           &local_ipaddr, sizeof(struct in_addr));
@@ -769,6 +793,11 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
                    rtp_proxytable[freeidx].remote_ipaddr,
                    rtp_proxytable[freeidx].remote_port + 1);
 
+   /* try to find the matching socket for return path. This has to be done for
+    * both directions, the new socket and if one found, it must link back. */
+   i=match_socket(freeidx);
+   if (i>=0 && i<RTPPROXY_SIZE) j=match_socket(i);
+
    /* prepare FD set for next select operation */
    rtp_recreate_fdset();
 
@@ -901,6 +930,9 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
                    rtp_proxytable[i].remote_ipaddr,
                    rtp_proxytable[i].remote_port + 1);
          /* clean up */
+         if (rtp_proxytable[i].opposite_entry) {
+            rtp_proxytable[rtp_proxytable[i].opposite_entry-1].opposite_entry=0;
+         }
          memset(&rtp_proxytable[i], 0, sizeof(rtp_proxytable[0]));
          got_match=1;
       }
@@ -1004,13 +1036,15 @@ static void rtpproxy_kill( void ) {
 /*
  * match_socket
  * matches and cross connects two rtp_proxytable entries
- * (corresponds to the two data directions of one RTP stream)
+ * (corresponds to the two data directions of one RTP stream
+ * within one call).
+ * returns the matching rtp_proxytable index of -1 if not found.
  */
-static void match_socket (int rtp_proxytable_idx) {
+static int match_socket (int rtp_proxytable_idx) {
    int j;
    int rtp_direction = rtp_proxytable[rtp_proxytable_idx].direction;
+   int call_direction = rtp_proxytable[rtp_proxytable_idx].call_direction;
    int media_stream_no = rtp_proxytable[rtp_proxytable_idx].media_stream_no;
-/*chnnel   int channel = rtp_proxytable[rtp_proxytable_idx].channel;*/
    osip_call_id_t callid;
 
    callid.number = rtp_proxytable[rtp_proxytable_idx].callid_number;
@@ -1028,10 +1062,10 @@ static void match_socket (int rtp_proxytable_idx) {
        * - different client ID
        */
       if ( (rtp_proxytable[j].rtp_rx_sock != 0) &&
-           (compare_callid(&callid, &cid) == STS_SUCCESS) &&
-           (media_stream_no == rtp_proxytable[j].media_stream_no) &&
-           (rtp_direction != rtp_proxytable[j].direction) /* channel: &&
-           (channel == rtp_proxytable[j].channel)*/ ) {
+           (compare_callid(&callid, &cid) == STS_SUCCESS) &&		// same Call-ID
+           (call_direction == rtp_proxytable[j].call_direction) &&	// same Call direction
+           (media_stream_no == rtp_proxytable[j].media_stream_no) &&	// same stream
+           (rtp_direction != rtp_proxytable[j].direction) ) {		// opposite RTP dir
          char remip1[16], remip2[16];
          char lclip1[16], lclip2[16];
          
@@ -1044,6 +1078,9 @@ static void match_socket (int rtp_proxytable_idx) {
          strcpy(remip2, utils_inet_ntoa(rtp_proxytable[rtp_proxytable_idx].remote_ipaddr));
          strcpy(lclip2, utils_inet_ntoa(rtp_proxytable[rtp_proxytable_idx].local_ipaddr));
 
+         rtp_proxytable[rtp_proxytable_idx].opposite_entry=j+1;
+         rtp_proxytable[j].opposite_entry=rtp_proxytable_idx+1;
+
          DEBUGC(DBCLASS_RTP, "connected entry %i (fd=%i, %s:%i->%s:%i) <-> entry %i (fd=%i, %s:%i->%s:%i)",
                              j, rtp_proxytable[j].rtp_rx_sock,
                              lclip1, rtp_proxytable[j].local_port,
@@ -1056,6 +1093,8 @@ static void match_socket (int rtp_proxytable_idx) {
          break;
       }
    }
+   if (j >= RTPPROXY_SIZE) j= -1;
+   return j;
 }
 
 
