@@ -32,7 +32,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: security.c,v 1.24 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: security.c 443 2010-01-07 11:31:56Z hb9xar $";
 
 /*
  * do security and integrity checks on the received packet
@@ -59,6 +59,8 @@ int security_check_raw(char *sip_buffer, size_t size) {
     * empiric: a line should not be longer than 256 characters
     * (libosip may die with "virtual memory exhausted" otherwise)
     * Ref: protos test suite c07-sip-r2.jar, test case 203
+    * !! Contact records may all come in one single line, getting QUITE long...
+    *    especially on TCP.
     */
    for (p1=sip_buffer; (p1+SEC_MAXLINELEN) < (sip_buffer+size); p1=p2+1) {
       p2=strchr(p1, 10);
