@@ -21,7 +21,7 @@
 #include <osipparser2/osip_parser.h>
 #include <log.h>
 
-static char const ident[]="$Id: sip_layer.c,v 1.8 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: sip_layer.c 458 2010-03-29 17:41:39Z hb9xar $";
 
 /*
  * This file contains wrapper functions to call the osip2_ library.
@@ -36,23 +36,39 @@ int sip_message_parse(osip_message_t * sip, const char *buf, size_t len) {
 
 int sip_message_to_str(osip_message_t * sip, char **dest, size_t *len) {
    int sts;
+   /* check params */
+   if ((len == NULL) || (dest == NULL)) return -1;
+
+   *len=0;
+
    sts = osip_message_to_str(sip, dest, len);
-   /*
-    * NULL termination (libosip2-2.2.0 does NOT do this properly,
-    * there is always one byte too much :-( )
-    */
-   (*dest)[*len]='\0';
+   if (sts == 0) {
+      /*
+       * NULL termination (libosip2-2.2.0 does NOT do this properly,
+       * there is always one byte too much :-( )
+       */
+      if (*len >= 0) (*dest)[*len]='\0';
+   }
+
    return sts;
 }
 
 int sip_body_to_str(const osip_body_t * body, char **dest, size_t *len) {
    int sts;
+   /* check params */
+   if ((len == NULL) || (dest == NULL)) return -1;
+
+   *len=0;
+
    sts = osip_body_to_str(body, dest, len);
-   /*
-    * NULL termination (libosip2-2.2.0 does NOT do this properly,
-    * there is always one byte too much :-( )
-    */
-   (*dest)[*len]='\0';
+   if (sts == 0) {
+      /*
+       * NULL termination (libosip2-2.2.0 does NOT do this properly,
+       * there is always one byte too much :-( )
+       */
+      if (*len >= 0) (*dest)[*len]='\0';
+   }
+
    return sts;
 }
 
