@@ -42,7 +42,7 @@
 #include "digcalc.h"
 #include "log.h"
 
-static char const ident[]="$Id: sip_utils.c,v 1.59 2008/07/22 11:17:49 hb9xar Exp $";
+static char const ident[]="$Id: sip_utils.c 467 2011-01-09 13:10:27Z hb9xar $";
 
 
 /* configuration storage */
@@ -428,7 +428,7 @@ int is_sipuri_local (sip_ticket_t *ticket) {
       /* need name resolution */
       sts=get_ip_by_host(sip->req_uri->host, &addr_uri);
       if (sts == STS_FAILURE) {
-         DEBUGC(DBCLASS_PROXY, "sip_gen_response: cannot resolve via [%s]",
+         DEBUGC(DBCLASS_PROXY, "sip_gen_response: cannot resolve request uri [%s]",
                 sip->req_uri->host);
          return STS_FALSE;
       }
@@ -578,10 +578,17 @@ int sip_add_myvia (sip_ticket_t *ticket, int interface) {
    sts = sip_calculate_branch_id(ticket, branch_id);
 
    myaddr=utils_inet_ntoa(addr);
-   sprintf(tmp, "SIP/2.0/UDP %s:%i;branch=%s%s",
-           myaddr, configuration.sip_listen_port,
-           branch_id,
-           (add_rport)? ";rport":"");
+   if (ticket->protocol == PROTO_UDP) {
+      sprintf(tmp, "SIP/2.0/UDP %s:%i;branch=%s%s",
+              myaddr, configuration.sip_listen_port,
+              branch_id,
+              (add_rport)? ";rport":"");
+   } else {
+      sprintf(tmp, "SIP/2.0/TCP %s:%i;branch=%s%s",
+              myaddr, configuration.sip_listen_port,
+              branch_id,
+              (add_rport)? ";rport":"");
+   }
 
    DEBUGC(DBCLASS_BABBLE,"adding VIA:%s",tmp);
 
@@ -697,6 +704,11 @@ int sip_rewrite_contact (sip_ticket_t *ticket, int direction) {
             osip_uri_clone(urlmap[i].true_url, &contact->url);
          }
 
+         /* add transport=tcp parameter if TCP */
+         if (ticket->protocol == PROTO_TCP) {
+            osip_uri_set_transport_tcp(contact->url);
+         }
+
          osip_list_add(&(sip_msg->contacts),contact,j);
          replaced=1;
       }
@@ -970,7 +982,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    struct sockaddr_in *from;
    osip_message_t *request;
    osip_message_t *response;
-   struct in_addr tmp_addr;
+   struct in_addr tmp_addr, tmp_addr2;
 
    from=&ticket->from;
    request=ticket->sipmsg;
@@ -979,9 +991,22 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
 
    ticket->direction = DIRTYP_UNKNOWN;
 
+   /* Search order is as follows:
+    * - "from" IP address one of our registered local UAs
+    * - "To:" SIP header (Request) directed to internal UA
+    *   or
+    *   "From:" SIP header (Response) coming from internal UA
+    * - SIP URI matches one of the registered local UAs
+    * - for Responses: check for bottommost "Via:" header to be
+    *   one of our registered local UA IPs
+    * - "from" IP == 127.0.0.1 || inbound_IP || outbound IP 
+    *
+    * The first successful match is taken.
+    */
+
    /*
     * did I receive the telegram from a REGISTERED host?
-    * -> it must be an OUTGOING request
+    * -> it must be an OUTGOING request/response
     */
    for (i=0; i<URLMAP_SIZE; i++) {
       if (urlmap[i].active == 0) continue;
@@ -1002,9 +1027,11 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
       }
    }
 
+
    /*
     * is the telegram directed to an internally registered host?
-    * -> it must be an INCOMING request
+    * -> likely to be an INCOMING request/response
+    * check for a match on the To: header  first
     */
    if (type == DIRTYP_UNKNOWN) {
       for (i=0; i<URLMAP_SIZE; i++) {
@@ -1017,20 +1044,12 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
 
          /* So this means, that we must check the SIP URI supplied with the
           * INVITE method, as this points to the real wanted target.
-          * Q: does there exist a situation where the SIP URI itself does
-          *    point to "somewhere" but the To: points to the correct UA?
-          * So for now, we just look at both of them (SIP URI and To: header)
+          * First we will try to match on the To: and From: headers
+          * If nothing is found here, we try again with SIP URI futher down.
           */
 
          if (MSG_IS_REQUEST(ticket->sipmsg)) {
             /* REQUEST */
-            /* incoming request (SIP URI == 'masq') || ((SIP URI == 'reg') && !REGISTER)*/
-            if ((compare_url(request->req_uri, urlmap[i].masq_url)==STS_SUCCESS) ||
-                (!MSG_IS_REGISTER(request) &&
-                 (compare_url(request->req_uri, urlmap[i].reg_url)==STS_SUCCESS))) {
-               type=REQTYP_INCOMING;
-               break;
-            }
             /* incoming request ('to' == 'masq') || (('to' == 'reg') && !REGISTER)*/
             if ((compare_url(request->to->url, urlmap[i].masq_url)==STS_SUCCESS) ||
                 (!MSG_IS_REGISTER(request) &&
@@ -1050,17 +1069,37 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
       } /* for i */
    } /* if type == DIRTYP_UNKNOWN */
 
-   if (MSG_IS_RESPONSE(ticket->sipmsg)) {
-      /* &&&& Open Issue &&&&
-         it has been seen with cross-provider calls that the FROM may be 'garbled'
-         (e.g 1393xxx@proxy01.sipphone.com for calls made sipphone -> FWD)
-         How can we deal with this? Should I take into consideration the 'Via'
-         headers? This is the only clue I have, pointing to the *real* UA.
-         Maybe I should put in a 'siproxd' ftag value to recognize it as a header
-         inserted by myself
-      */
-      if ((type == DIRTYP_UNKNOWN) && 
-          (!osip_list_eol(&(response->vias), 0))) {
+
+   /*
+    * nothing found yet? 
+    * -> likely to be an INCOMING request/response
+    * check for a match on the SIP URI (requests only)
+    */
+   if ((type == DIRTYP_UNKNOWN) && (MSG_IS_REQUEST(ticket->sipmsg))) {
+      for (i=0; i<URLMAP_SIZE; i++) {
+         if (urlmap[i].active == 0) continue;
+         /* incoming request (SIP URI == 'masq') || ((SIP URI == 'reg') && !REGISTER)*/
+         if ((compare_url(request->req_uri, urlmap[i].masq_url)==STS_SUCCESS) ||
+             (!MSG_IS_REGISTER(request) &&
+              (compare_url(request->req_uri, urlmap[i].reg_url)==STS_SUCCESS))) {
+            type=REQTYP_INCOMING;
+            break;
+         }
+      } /* for i */
+   } /* if type == DIRTYP_UNKNOWN */
+
+
+   /* &&&& Open Issue &&&&
+    * it has been seen with cross-provider calls that the FROM may be 'garbled'
+    * (e.g 1393xxx@proxy01.sipphone.com for calls made sipphone -> FWD)
+    * How can we deal with this? Should I take into consideration the 'Via'
+    * headers? This is the only clue I have, pointing to the *real* UA.
+    * Maybe I should put in a 'siproxd' ftag value to recognize it as a header
+    * inserted by myself
+    */
+   if ((type == DIRTYP_UNKNOWN) && 
+       (!osip_list_eol(&(response->vias), 0))) {
+      if (MSG_IS_RESPONSE(ticket->sipmsg)) {
          osip_via_t *via;
          struct in_addr addr_via, addr_myself;
          int port_via, port_ua;
@@ -1106,27 +1145,33 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
                }
             } /* for i */
          }
-      } /* if type == DIRTYP_UNKNOWN */
-   } /* is response */
+      } /* is response */
+   } /* if type == DIRTYP_UNKNOWN */
+
 
    /*
     * if the telegram is received from 127.0.0.1 use my inbound IP as sender,
     * this likely is a locally REDIRECTED/DNATed (by iptables) packet.
     * So it is a local UA.
+    * Also, my own outbound address is considered to be redirected traffic
     * Example Scenario:
     * Softphone(or PBX) running on the same host as siproxd is running.
-    * Using iptables, you do a REDIRECT of outgoping SIP traffix of the
+    * Using iptables, you do a REDIRECT of outgoing SIP traffic of the
     * PBX to be passed to siproxd.
     */
-   sts=get_interface_ip(IF_INBOUND, &tmp_addr);
-   if ((htonl(from->sin_addr.s_addr) == INADDR_LOOPBACK) ||
-       (from->sin_addr.s_addr == tmp_addr.s_addr)) {
-      if (MSG_IS_REQUEST(ticket->sipmsg)) {
-         type=REQTYP_OUTGOING;
-      } else {
-         type=RESTYP_OUTGOING;
+   if (type == DIRTYP_UNKNOWN) {
+      sts=get_interface_ip(IF_INBOUND, &tmp_addr);
+      sts=get_interface_ip(IF_OUTBOUND, &tmp_addr2);
+      if ((htonl(from->sin_addr.s_addr) == INADDR_LOOPBACK) ||
+	  (from->sin_addr.s_addr == tmp_addr.s_addr) ||
+	  (from->sin_addr.s_addr == tmp_addr2.s_addr)) {
+	 if (MSG_IS_REQUEST(ticket->sipmsg)) {
+            type=REQTYP_OUTGOING;
+	 } else {
+            type=RESTYP_OUTGOING;
+	 }
       }
-   }
+   } /* if type == DIRTYP_UNKNOWN */
 
 
    if (type == DIRTYP_UNKNOWN) {
@@ -1343,3 +1388,83 @@ DEBUGC(DBCLASS_PROXY, "tmp+myidentlen=[%s], FromTag=[%s]",
 }
 
 
+/*
+ * SIP_ADD_RECEIVED_PARAM
+ *
+ * Add a received parameter to the topmost VIA header (IP and port)
+ * 
+ * RETURNS
+ *	STS_SUCCESS on success
+ */
+int sip_add_received_param(sip_ticket_t *ticket){
+   osip_via_t *via;
+   char tmp[6];
+
+   DEBUGC(DBCLASS_PROXY,"adding received= param to topmost via");
+   via = osip_list_get (&(ticket->sipmsg->vias), 0);
+   
+   /* set rport=xxx;received=1.2.3.4 */
+   snprintf(tmp, sizeof(tmp), "%i", ntohs(ticket->from.sin_port));
+   osip_via_param_add(via,osip_strdup("rport"),osip_strdup(tmp));
+
+   osip_via_param_add(via,osip_strdup("received"),
+                      osip_strdup(utils_inet_ntoa(ticket->from.sin_addr)));
+
+   return STS_SUCCESS;
+}
+
+
+/*
+ * SIP_GET_RECEIVED_PARAM
+ *
+ * Get a received parameter from the topmost VIA header (IP and port)
+ * 
+ * RETURNS
+ *	STS_SUCCESS on success
+ */
+int  sip_get_received_param(sip_ticket_t *ticket,
+                            struct in_addr *dest, int *port) {
+   osip_via_t *via;
+   osip_generic_param_t *received=NULL;
+   osip_generic_param_t *rport=NULL;
+   int sts;
+
+   DEBUGC(DBCLASS_PROXY,"searching received= param in topmost via");
+   via = osip_list_get (&(ticket->sipmsg->vias), 0);
+
+   osip_via_param_get_byname (via, "received", &received);
+   osip_via_param_get_byname (via, "rport",    &rport);
+
+   if (received && rport && received->gvalue && rport->gvalue) {
+      /* fetch the IP */
+      sts = get_ip_by_host(received->gvalue, dest);
+      if (sts != STS_SUCCESS) return STS_FAILURE;
+      
+      /* fetch the port number */
+      *port = atoi(rport->gvalue);
+      if ((*port <=0) || (*port >=65536)) return STS_FAILURE;
+
+      /* If TCP, then validate first if an existing connection is in the cache.
+       * If not, do not use this - a new conection must be established! */
+      if (ticket->protocol == PROTO_TCP) {
+         struct sockaddr_in addr;
+         addr.sin_family = AF_INET;
+         memcpy(&addr.sin_addr, dest, sizeof(struct in_addr));
+         addr.sin_port= htons(*port);
+         sts = tcp_find(addr);
+         if (sts < 0) {
+            DEBUGC(DBCLASS_BABBLE, "IP: %s, port: %i not found in cache",
+                   utils_inet_ntoa(*dest), *port);
+            return STS_FAILURE;
+         }
+      }
+
+      /* found, return */
+      DEBUGC(DBCLASS_BABBLE, "IP:%s, port:%i is ok to be reused",
+             utils_inet_ntoa(*dest), *port);
+      return STS_SUCCESS;
+   }
+
+   /* not found */
+   return STS_FAILURE;
+}
