@@ -34,6 +34,10 @@
 #include <net/if.h>
 #include <sys/ioctl.h>
 
+#ifdef HAVE_GETIFADDRS
+# include <ifaddrs.h>
+#endif
+
 #ifdef _SOLARIS2
 # include <sys/sockio.h>
 #endif
@@ -46,7 +50,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: utils.c,v 1.55 2009/07/30 17:02:28 hb9xar Exp $";
+static char const ident[]="$Id: utils.c 481 2011-06-12 17:53:48Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -401,11 +405,9 @@ int  get_interface_real_ip(int interface, struct in_addr *retaddr) {
  * STS_FAILURE if interface is DOWN or other problem
  */
 int get_ip_by_ifname(char *ifname, struct in_addr *retaddr) {
-   struct ifreq ifr;
-   struct sockaddr_in *sin = (struct sockaddr_in *)&ifr.ifr_addr;
-   int sockfd;
+   struct in_addr ifaddr; /* resulting IP */
    int i, j;
-   int ifflags, isup;
+   int ifflags=0, isup=0;
    time_t t;
    static struct {
       time_t timestamp;
@@ -414,6 +416,14 @@ int get_ip_by_ifname(char *ifname, struct in_addr *retaddr) {
       char ifname[IFNAME_SIZE+1];
    } ifaddr_cache[IFADR_CACHE_SIZE];
    static int cache_initialized=0;
+#ifdef HAVE_GETIFADDRS
+   struct ifaddrs *ifa;
+   struct ifaddrs *ifa_list;
+#else
+   struct ifreq ifr;
+   struct sockaddr_in *sin = (struct sockaddr_in *)&ifr.ifr_addr;
+   int sockfd;
+#endif
 
    if (ifname == NULL) {
       WARN("get_ip_by_ifname: got NULL ifname passed - please check config"
@@ -457,38 +467,79 @@ int get_ip_by_ifname(char *ifname, struct in_addr *retaddr) {
    } /* for i */
 
    /* not found in cache, go and get it */
-   memset(&ifr, 0, sizeof(ifr));
 
-   if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-      ERROR("Error in socket: %s\n",strerror(errno));
+#ifdef HAVE_GETIFADDRS
+   if (getifaddrs(&ifa_list)) {
+      ERROR("Error in getifaddrs: %s",strerror(errno));
       return STS_FAILURE;
    }
 
+   i=0; /* use "found" marker */
+   for (ifa = ifa_list; ifa != NULL; ifa = ifa->ifa_next) {
+      DEBUGC(DBCLASS_BABBLE,"getifaddrs - %s / %s, ifa_addr=%p, addrfamily=%i",
+             ifname, ifa->ifa_name,ifa->ifa_addr,
+             (ifa->ifa_addr)?ifa->ifa_addr->sa_family:-1);
+
+      if (ifa && ifa->ifa_name && ifa->ifa_addr && 
+          ifa->ifa_addr->sa_family == AF_INET &&
+          strcmp(ifa->ifa_name, ifname) == 0) {
+         /* found the entry */
+         i=1;
+         memcpy(&ifaddr, &((struct sockaddr_in*)ifa->ifa_addr)->sin_addr, sizeof(struct in_addr));
+         ifflags=ifa->ifa_flags;
+         DEBUGC(DBCLASS_BABBLE,"getifaddrs - MATCH, sin_addr=%s",
+                utils_inet_ntoa(ifaddr));
+         break;
+      }
+   }
+   freeifaddrs(ifa_list);
+
+   if (i==0) {
+      DEBUGC(DBCLASS_DNS,"Interface %s not found.", ifname);
+      return STS_FAILURE;
+   }
+
+#else
+   memset(&ifr, 0, sizeof(ifr));
+
+   if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+      ERROR("Error in socket: %s",strerror(errno));
+      return STS_FAILURE;
+   }
+
+/*&&&*/DEBUGC(DBCLASS_BABBLE,"&&&6 ifname=0x%p",ifname);
    strcpy(ifr.ifr_name, ifname);
    sin->sin_family = AF_INET;
 
    /* get interface flags */
    if(ioctl(sockfd, SIOCGIFFLAGS, &ifr) != 0) {
-      ERROR("Error in ioctl SIOCGIFFLAGS: %s [%s]\n",
+      ERROR("Error in ioctl SIOCGIFFLAGS: %s [%s]",
             strerror(errno), ifname);
       close(sockfd);
       return STS_FAILURE;
    } 
    ifflags=ifr.ifr_flags;
+/*&&&*/DEBUGC(DBCLASS_BABBLE,"&&&7 ifname=0x%p",ifname);
 
    /* get address */
    if(ioctl(sockfd, SIOCGIFADDR, &ifr) != 0) {
-      ERROR("Error in ioctl SIOCGIFADDR: %s (interface %s)\n",
+      ERROR("Error in ioctl SIOCGIFADDR: %s (interface %s)",
       strerror(errno), ifname);
       close(sockfd);
       return STS_FAILURE;
-   } 
+   }
+
+/*&&&*/DEBUGC(DBCLASS_BABBLE,"&&&8 ifname=0x%p",ifname);
+   memcpy(&ifaddr, &sin->sin_addr, sizeof(struct in_addr));
+/*&&&*/DEBUGC(DBCLASS_BABBLE,"&&&9 ifname=0x%p",ifname);
+   close(sockfd);
+#endif
 
    if (ifflags & IFF_UP) isup=1;
    else isup=0;
 
    DEBUGC(DBCLASS_DNS, "get_ip_by_ifname: if %s has IP:%s (flags=%x) %s",
-          ifname, utils_inet_ntoa(sin->sin_addr), ifflags,
+          ifname, utils_inet_ntoa(ifaddr), ifflags,
           (isup)? "UP":"DOWN");
 
    /*
@@ -513,12 +564,11 @@ int get_ip_by_ifname(char *ifname, struct in_addr *retaddr) {
    memset(&ifaddr_cache[i], 0, sizeof(ifaddr_cache[0]));
    strncpy(ifaddr_cache[i].ifname, ifname, IFNAME_SIZE);
    ifaddr_cache[i].timestamp=t;
-   memcpy(&ifaddr_cache[i].ifaddr, &sin->sin_addr, sizeof(sin->sin_addr));
+   memcpy(&ifaddr_cache[i].ifaddr, &ifaddr, sizeof(struct in_addr));
    ifaddr_cache[i].isup=isup;
 
-   if (retaddr) memcpy(retaddr, &sin->sin_addr, sizeof(sin->sin_addr));
+   if (retaddr) memcpy(retaddr, &ifaddr, sizeof(struct in_addr));
 
-   close(sockfd);
    return (isup)? STS_SUCCESS : STS_FAILURE;
 }
 
@@ -540,7 +590,7 @@ char *utils_inet_ntoa(struct in_addr in) {
 #if defined(HAVE_INET_NTOP)
    static char string[INET_ADDRSTRLEN];
    if ((inet_ntop(AF_INET, &in, string, INET_ADDRSTRLEN)) == NULL) {
-      ERROR("inet_ntop() failed: %s\n",strerror(errno));
+      ERROR("inet_ntop() failed: %s",strerror(errno));
       string[0]='\0';
    }
    return string;
